@@ -30,6 +30,7 @@
 #include "torture/ldap/proto.h"
 #include "libcli/auth/libcli_auth.h"
 #include "../lib/crypto/crypto.h"
+#include "../libcli/drsuapi/drsuapi.h"
 #include "auth/credentials/credentials.h"
 #include "libcli/auth/libcli_auth.h"
 #include "auth/gensec/gensec.h"
@@ -273,7 +274,12 @@ static bool test_GetInfo(struct torture_context *tctx, struct DsSyncTest *ctx)
 	struct cldap_socket *cldap;
 	struct cldap_netlogon search;
 
-	cldap = cldap_socket_init(ctx, tctx->ev, lp_iconv_convenience(tctx->lp_ctx));
+	status = cldap_socket_init(ctx, NULL, NULL, NULL, &cldap);
+	if (!NT_STATUS_IS_OK(status)) {
+		printf("failed to setup cldap socket - %s\n",
+			nt_errstr(status));
+		return false;
+	}
 
 	r.in.bind_handle		= &ctx->admin.drsuapi.bind_handle;
 	r.in.level			= 1;
@@ -311,7 +317,7 @@ static bool test_GetInfo(struct torture_context *tctx, struct DsSyncTest *ctx)
 	search.in.acct_control = -1;
 	search.in.version		= NETLOGON_NT_VERSION_5 | NETLOGON_NT_VERSION_5EX;
 	search.in.map_response = true;
-	status = cldap_netlogon(cldap, ctx, &search);
+	status = cldap_netlogon(cldap, lp_iconv_convenience(tctx->lp_ctx), ctx, &search);
 	if (!NT_STATUS_IS_OK(status)) {
 		const char *errstr = nt_errstr(status);
 		ctx->site_name = talloc_asprintf(ctx, "%s", "Default-First-Site-Name");
@@ -333,119 +339,6 @@ static bool test_GetInfo(struct torture_context *tctx, struct DsSyncTest *ctx)
 	return ret;
 }
 
-static DATA_BLOB decrypt_blob(TALLOC_CTX *mem_ctx,
-			      const DATA_BLOB *gensec_skey,
-			      bool rcrypt,
-			      struct drsuapi_DsReplicaObjectIdentifier *id,
-			      uint32_t rid,
-			      const DATA_BLOB *buffer)
-{
-	DATA_BLOB confounder;
-	DATA_BLOB enc_buffer;
-
-	struct MD5Context md5;
-	uint8_t _enc_key[16];
-	DATA_BLOB enc_key;
-
-	DATA_BLOB dec_buffer;
-
-	uint32_t crc32_given;
-	uint32_t crc32_calc;
-	DATA_BLOB checked_buffer;
-
-	DATA_BLOB plain_buffer;
-
-	/*
-	 * the combination "c[3] s[1] e[1] d[0]..."
-	 * was successful!!!!!!!!!!!!!!!!!!!!!!!!!!
-	 */
-
-	/* 
-	 * the first 16 bytes at the beginning are the confounder
-	 * followed by the 4 byte crc32 checksum
-	 */
-	if (buffer->length < 20) {
-		return data_blob_const(NULL, 0);
-	}
-	confounder = data_blob_const(buffer->data, 16);
-	enc_buffer = data_blob_const(buffer->data + 16, buffer->length - 16);
-
-	/* 
-	 * build the encryption key md5 over the session key followed
-	 * by the confounder
-	 * 
-	 * here the gensec session key is used and
-	 * not the dcerpc ncacn_ip_tcp "SystemLibraryDTC" key!
-	 */
-	enc_key = data_blob_const(_enc_key, sizeof(_enc_key));
-	MD5Init(&md5);
-	MD5Update(&md5, gensec_skey->data, gensec_skey->length);
-	MD5Update(&md5, confounder.data, confounder.length);
-	MD5Final(enc_key.data, &md5);
-
-	/*
-	 * copy the encrypted buffer part and 
-	 * decrypt it using the created encryption key using arcfour
-	 */
-	dec_buffer = data_blob_talloc(mem_ctx, enc_buffer.data, enc_buffer.length);
-	if (!dec_buffer.data) {
-		return data_blob_const(NULL, 0);
-	}
-	arcfour_crypt_blob(dec_buffer.data, dec_buffer.length, &enc_key);
-
-	/* 
-	 * the first 4 byte are the crc32 checksum
-	 * of the remaining bytes
-	 */
-	crc32_given = IVAL(dec_buffer.data, 0);
-	crc32_calc = crc32_calc_buffer(dec_buffer.data + 4 , dec_buffer.length - 4);
-	if (crc32_given != crc32_calc) {
-		DEBUG(0,("CRC32: given[0x%08X] calc[0x%08X]\n",
-		      crc32_given, crc32_calc));
-		return data_blob_const(NULL, 0);
-	}
-	checked_buffer = data_blob_talloc(mem_ctx, dec_buffer.data + 4, dec_buffer.length - 4);
-	if (!checked_buffer.data) {
-		return data_blob_const(NULL, 0);
-	}
-
-	/*
-	 * some attributes seem to be in a usable form after this decryption
-	 * (supplementalCredentials, priorValue, currentValue, trustAuthOutgoing,
-	 *  trustAuthIncoming, initialAuthOutgoing, initialAuthIncoming)
-	 * At least supplementalCredentials contains plaintext
-	 * like "Primary:Kerberos" (in unicode form)
-	 *
-	 * some attributes seem to have some additional encryption
-	 * dBCSPwd, unicodePwd, ntPwdHistory, lmPwdHistory
-	 *
-	 * it's the sam_rid_crypt() function, as the value is constant,
-	 * so it doesn't depend on sessionkeys.
-	 */
-	if (rcrypt) {
-		uint32_t i, num_hashes;
-
-		if ((checked_buffer.length % 16) != 0) {
-			return data_blob_const(NULL, 0);
-		}
-
-		plain_buffer = data_blob_talloc(mem_ctx, checked_buffer.data, checked_buffer.length);
-		if (!plain_buffer.data) {
-			return data_blob_const(NULL, 0);
-		}
-			
-		num_hashes = plain_buffer.length / 16;
-		for (i = 0; i < num_hashes; i++) {
-			uint32_t offset = i * 16;
-			sam_rid_crypt(rid, checked_buffer.data + offset, plain_buffer.data + offset, 0);
-		}
-	} else {
-		plain_buffer = checked_buffer;
-	}
-
-	return plain_buffer;
-}
-
 static void test_analyse_objects(struct torture_context *tctx, 
 				 struct DsSyncTest *ctx,
 				 const DATA_BLOB *gensec_skey,
@@ -476,6 +369,7 @@ static void test_analyse_objects(struct torture_context *tctx,
 		}
 
 		for (i=0; i < cur->object.attribute_ctr.num_attributes; i++) {
+			WERROR werr;
 			const char *name = NULL;
 			bool rcrypt = false;
 			DATA_BLOB *enc_data = NULL;
@@ -544,9 +438,13 @@ static void test_analyse_objects(struct torture_context *tctx,
 			enc_data = attr->value_ctr.values[0].blob;
 			ZERO_STRUCT(plain_data);
 
-			plain_data = decrypt_blob(ctx, gensec_skey, rcrypt,
-						  cur->object.identifier, rid,
-						  enc_data);
+			werr = drsuapi_decrypt_attribute_value(ctx, gensec_skey, rcrypt,
+							       rid,
+							       enc_data, &plain_data);
+			if (!W_ERROR_IS_OK(werr)) {
+				DEBUG(0, ("Failed to decrypt %s\n", name));
+				continue;
+			}
 			if (!dn_printed) {
 				object_id++;
 				DEBUG(0,("DN[%u] %s\n", object_id, dn));
